@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 # shellcheck disable=SC2155
 
 _TOTAL_TESTS_COUNT=0
@@ -106,7 +106,7 @@ function console_results::print_execution_time() {
     return
   fi
 
-  local time=$(printf "%.0f" "$(clock::total_runtime_in_milliseconds)")
+  local time=$(clock::total_runtime_in_milliseconds | awk '{printf "%.0f", $1}')
 
   if [[ "$time" -lt 1000 ]]; then
     printf "${_COLOR_BOLD}%s${_COLOR_DEFAULT}\n" \
@@ -116,7 +116,7 @@ function console_results::print_execution_time() {
 
   local time_in_seconds=$(( time / 1000 ))
   local remainder_ms=$(( time % 1000 ))
-  local formatted_seconds=$(printf "%.2f" "$time_in_seconds.$remainder_ms")
+  local formatted_seconds=$(echo "$time_in_seconds.$remainder_ms" | awk '{printf "%.0f", $1}')
 
   printf "${_COLOR_BOLD}%s${_COLOR_DEFAULT}\n" \
     "Time taken: $formatted_seconds s"
