@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 set -euo pipefail
 
 function dependencies::has_perl() {
@@ -32,3 +32,11 @@ function dependencies::has_curl() {
 function dependencies::has_wget() {
   command -v wget >/dev/null 2>&1
 }
+
+function dependencies::has_python() {
+  command -v python >/dev/null 2>&1
+}
+
+function dependencies::has_node() {
+  command -v node >/dev/null 2>&1
+}
