@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 declare -r BASHUNIT_GIT_REPO="https://github.com/TypedDevs/bashunit"
 
@@ -8,14 +8,20 @@ declare -r BASHUNIT_GIT_REPO="https://github.com/TypedDevs/bashunit"
 # @return string Eg: "Some logic camelCase"
 #
 function helper::normalize_test_function_name() {
-  local original_function_name="${1-}"
+  local original_fn_name="${1-}"
+  local interpolated_fn_name="${2-}"
+
+  if [[ -n "${interpolated_fn_name-}" ]]; then
+    original_fn_name="$interpolated_fn_name"
+  fi
+
   local result
 
   # Remove the first "test_" prefix, if present
-  result="${original_function_name#test_}"
+  result="${original_fn_name#test_}"
   # If no "test_" was removed (e.g., "testFoo"), remove the "test" prefix
-  if [[ "$result" == "$original_function_name" ]]; then
-    result="${original_function_name#test}"
+  if [[ "$result" == "$original_fn_name" ]]; then
+    result="${original_fn_name#test}"
   fi
   # Replace underscores with spaces
   result="${result//_/ }"
@@ -25,6 +31,29 @@ function helper::normalize_test_function_name() {
   echo "$result"
 }
 
+function helper::escape_single_quotes() {
+  local value="$1"
+  # shellcheck disable=SC1003
+  echo "${value//\'/'\'\\''\'}"
+}
+
+function helper::interpolate_function_name() {
+  local function_name="$1"
+  shift
+  local args=("$@")
+  local result="$function_name"
+
+  for ((i=0; i<${#args[@]}; i++)); do
+    local placeholder="::$((i+1))::"
+    # shellcheck disable=SC2155
+    local value="$(helper::escape_single_quotes "${args[$i]}")"
+    value="'$value'"
+    result="${result//${placeholder}/${value}}"
+  done
+
+  echo "$result"
+}
+
 function helper::check_duplicate_functions() {
   local script="$1"
 
@@ -96,11 +125,12 @@ function helper::unset_if_exists() {
 function helper::find_files_recursive() {
   ## Remove trailing slash using parameter expansion
   local path="${1%%/}"
+  local pattern="${2:-*[tT]est.sh}"
 
   if [[ "$path" == *"*"* ]]; then
-    eval find "$path" -type f -name '*[tT]est.sh' | sort | uniq
+    eval find "$path" -type f -name "$pattern" | sort -u
   elif [[ -d "$path" ]]; then
-    find "$path" -type f -name '*[tT]est.sh' | sort | uniq
+    find "$path" -type f -name "$pattern" | sort -u
   else
     echo "$path"
   fi
@@ -122,17 +152,17 @@ function helper::normalize_variable_name() {
 function helper::get_provider_data() {
   local function_name="$1"
   local script="$2"
-  local data_provider_function
 
   if [[ ! -f "$script" ]]; then
     return
   fi
 
-  data_provider_function=$(\
-    grep -B 1 "function $function_name()" "$script" |\
-    grep "# data_provider " |\
-    sed -E -e 's/\ *# data_provider (.*)$/\1/g'\
-    || true
+  local data_provider_function
+  data_provider_function=$(
+    # shellcheck disable=SC1087
+    grep -B 2 -E "function[[:space:]]+$function_name[[:space:]]*\(\)" "$script" 2>/dev/null | \
+    grep -E "^[[:space:]]*# *@?data_provider[[:space:]]+" | \
+    sed -E 's/^[[:space:]]*# *@?data_provider[[:space:]]+//' || true
   )
 
   if [[ -n "$data_provider_function" ]]; then
@@ -161,16 +191,52 @@ function helpers::get_latest_tag() {
 function helpers::find_total_tests() {
     local filter=${1:-}
     local files=("${@:2}")
+
+    if [[ ${#files[@]} -eq 0 ]]; then
+        echo 0
+        return
+    fi
+
     local total_count=0
+    local file
 
     for file in "${files[@]}"; do
-        local count
-        if [[ -n "$filter" ]]; then
-            count=$(grep -r -E "^\s*function\s+test.*$filter" "$file" --include=\*.sh 2>/dev/null | wc -l)
-        else
-            count=$(grep -r -E '^\s*function\s+test' "$file" --include=\*.sh 2>/dev/null | wc -l)
+        if [[ ! -f "$file" ]]; then
+            continue
         fi
-        total_count=$((total_count + count))
+
+        local file_count
+        file_count=$( (
+            # shellcheck source=/dev/null
+            source "$file"
+            local all_fn_names
+            all_fn_names=$(declare -F | awk '{print $3}')
+            local filtered_functions
+            filtered_functions=$(helper::get_functions_to_run "test" "$filter" "$all_fn_names") || true
+
+            local count=0
+            if [[ -n "$filtered_functions" ]]; then
+                # shellcheck disable=SC2206
+                # shellcheck disable=SC2207
+                local functions_to_run=($filtered_functions)
+                for fn_name in "${functions_to_run[@]}"; do
+                    local provider_data=()
+                    while IFS=" " read -r line; do
+                        provider_data+=("$line")
+                    done <<< "$(helper::get_provider_data "$fn_name" "$file")"
+
+                    if [[ "${#provider_data[@]}" -eq 0 ]]; then
+                        count=$((count + 1))
+                    else
+                        count=$((count + ${#provider_data[@]}))
+                    fi
+                done
+            fi
+
+            echo "$count"
+        ) )
+
+        total_count=$((total_count + file_count))
     done
 
     echo "$total_count"
@@ -194,3 +260,22 @@ function helper::load_test_files() {
 
   printf "%s\n" "${test_files[@]}"
 }
+
+function helper::load_bench_files() {
+  local filter=$1
+  local files=("${@:2}")
+
+  local bench_files=()
+
+  if [[ "${#files[@]}" -eq 0 ]]; then
+    if [[ -n "${BASHUNIT_DEFAULT_PATH}" ]]; then
+      while IFS='' read -r line; do
+        bench_files+=("$line")
+      done < <(helper::find_files_recursive "$BASHUNIT_DEFAULT_PATH" '*[bB]ench.sh')
+    fi
+  else
+    bench_files=("${files[@]}")
+  fi
+
+  printf "%s\n" "${bench_files[@]}"
+}
